/* BDF - Copyright (c) 2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.servlets;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.instruction.BdfInstruction;
import fr.exemole.bdfserver.api.interaction.Domain;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.api.users.BdfUserConstants;
import fr.exemole.bdfserver.conf.ConfConstants;
import fr.exemole.bdfserver.get.GetInstructionFactory;
import static fr.exemole.bdfserver.servlets.BDFWebapps.BDFSESSION_KEY;
import fr.exemole.bdfserver.servlets.instructions.AppInstruction;
import fr.exemole.bdfserver.servlets.instructions.DomainInstruction;
import fr.exemole.bdfserver.servlets.instructions.RunInstruction;
import fr.exemole.bdfserver.servlets.instructions.SessionInstruction;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class BDFUtils {

    private BDFUtils() {

    }

    public static String getRootUrl(HttpServletRequest request, String fichothequeName) {
        StringBuilder url = new StringBuilder();
        String scheme = request.getScheme();
        int port = request.getServerPort();
        if (port < 0) {
            port = 80; // Work around java.net.URL bug
        }
        url.append(scheme);
        url.append("://");
        url.append(request.getServerName());
        if ((scheme.equals("http") && (port != 80))
                || (scheme.equals("https") && (port != 443))) {
            url.append(':');
            url.append(port);
        }
        url.append(request.getContextPath());
        url.append("/");
        if (!fichothequeName.equals(ConfConstants.UNIQUE_NAME)) {
            url.append(fichothequeName);
            url.append("/");
        }
        return url.toString();
    }

    public static BdfUser getCurrentBdfUser(BdfServer bdfServer, HttpServletRequest request) {
        HttpSession session = request.getSession(false);
        if (session != null) {
            BDFSession bdfSession = (BDFSession) session.getAttribute(BDFSESSION_KEY);
            if (bdfSession != null) {
                BdfUser bdfUser = bdfSession.getBdfUser(bdfServer.getName());
                if (bdfUser != null) {
                    bdfUser.putParameter(BdfUserConstants.SESSION_LASTACCESSEDTIME, System.currentTimeMillis());
                    return bdfUser;
                }
            }
        }
        return null;
    }

    public static Domain parsePathInfo(String pathInfo) {
        String firstPart = pathInfo.toLowerCase();
        String secondPart = "";
        int length = pathInfo.length();
        for (int i = 0; i < length; i++) {
            char carac = pathInfo.charAt(i);
            boolean isSeparator = false;
            switch (carac) {
                case ':':
                case '_':
                case '-':
                    isSeparator = true;
                    break;
            }
            if (isSeparator) {
                secondPart = firstPart.substring(i + 1);
                firstPart = firstPart.substring(0, i);
                break;
            }
        }
        return new Domain(firstPart, secondPart);
    }

    public static BdfInstruction getDomainBdfInstruction(BdfServer bdfServer, RequestMap requestMap, Domain domain) {
        switch (domain.getFirstPart()) {
            case Domains.SESSION:
                return new SessionInstruction(bdfServer, requestMap);
            case Domains.RUN:
                return new RunInstruction(bdfServer, requestMap);
            case Domains.APP:
                return AppInstruction.build(bdfServer, requestMap, domain.getSecondPart());
            case Domains.ADMINISTRATION:
            case Domains.ALBUM:
            case Domains.CONFIGURATION:
            case Domains.CORPUS:
            case Domains.MAILING:
            case Domains.EXPORTATION:
            case Domains.EDITION:
            case Domains.SELECTION:
            case Domains.SPHERE:
            case Domains.THESAURUS:
            case Domains.MAIN:
            case Domains.MISC:
            case Domains.ADDENDA:
            case Domains.IMPORTATION:
            case Domains.PIOCHE:
            case Domains.EXTENSION:
                return new DomainInstruction(bdfServer, requestMap, domain);
            default:
                return null;
        }
    }

    public static BdfInstruction getBdfInstruction(BdfServer bdfServer, RequestMap requestMap, String pathInfo) {
        int idx = pathInfo.indexOf('/');
        if (idx == -1) {
            Domain domain = BDFUtils.parsePathInfo(pathInfo);
            return BDFUtils.getDomainBdfInstruction(bdfServer, requestMap, domain);
        } else {
            String first = pathInfo.substring(0, idx);
            String next = pathInfo.substring(idx + 1);
            return GetInstructionFactory.getBdfInstruction(bdfServer, requestMap, first, next);
        }
    }

}
