/* BdfServer_Multi - Copyright (c) 2021-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi.jsonproducers.admin;

import fr.exemole.bdfserver.multi.MultiUtils;
import fr.exemole.bdfserver.multi.api.Multi;
import fr.exemole.bdfserver.multi.api.MultiConstants;
import fr.exemole.bdfserver.multi.jsonproducers.MultiJsonProducer;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.synthesis.RedacteurSynthesis;
import fr.exemole.bdfserver.tools.synthesis.SphereSynthesis;
import java.text.ParseException;
import net.fichotheque.SubsetKey;
import net.mapeadores.util.json.JsonProperty;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class AdminJsonProducerFactory {

    private AdminJsonProducerFactory() {

    }

    public static MultiJsonProducer getJsonProducer(Multi multi, String json, CommandMessage commandMessage, RequestMap requestMap) {
        MessageLocalisation messageLocalisation = multi.getAdminMessageLocalisation();
        Lang lang = multi.getAdminLang();
        MultiJsonProducer jsonProducer = new MultiJsonProducer(true, messageLocalisation);
        if (commandMessage != null) {
            jsonProducer.setCommandMessage(commandMessage);
        }
        switch (json) {
            case MultiConstants.FICHOTHEQUE_ARRAY_JSON: {
                jsonProducer.add(new FichothequeArrayJsonProperty(multi, messageLocalisation, lang));
                break;
            }
            case MultiConstants.METADATA_JSON: {
                jsonProducer.add(new MetadataJsonProperty(multi));
                break;
            }
            case MultiConstants.PERSON_JSON: {
                try {
                    jsonProducer.add(getPersonJsonProperty(multi, requestMap));
                } catch (ErrorMessageException eme) {
                    jsonProducer.setCommandMessage(eme.getErrorMessage());
                }
                break;
            }
            case "":
            case MultiConstants.PING_JSON: {
                break;
            }
            default: {
                if (commandMessage == null) {
                    jsonProducer.setCommandMessage(LogUtils.error("_ error.unknown.json", json));
                }
            }
        }
        return jsonProducer;
    }

    private static JsonProperty getPersonJsonProperty(Multi multi, RequestMap requestMap) throws ErrorMessageException {
        if (!multi.isAuthentificationSharingAllowed()) {
            throw new ErrorMessageException("_ error.unsupported.sharingnotallowed");
        }
        String sphereName = requestMap.getParameter("sphere");
        if (sphereName == null) {
            throw BdfErrors.emptyMandatoryParameter("sphere");
        }
        String login = requestMap.getParameter("login");
        if (login == null) {
            throw BdfErrors.emptyMandatoryParameter("login");
        }
        SubsetKey sphereKey;
        try {
            sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereName.trim());
        } catch (ParseException pe) {
            throw new ErrorMessageException("_ error.unknown.sphere", sphereName);
        }
        SphereSynthesis sphereSynthesis = MultiUtils.getSphereSynthesis(multi, sphereKey);
        if (sphereSynthesis.getFichothequeSize() == 0) {
            throw new ErrorMessageException("_ error.unknown.sphere", sphereName);
        }
        RedacteurSynthesis redacteurSynthesis = sphereSynthesis.getRedacteurSynthesis(login);
        if (redacteurSynthesis == null) {
            throw new ErrorMessageException("_ error.unknown.redacteur", login);
        }
        return new PersonJsonProperty(multi, sphereSynthesis, redacteurSynthesis, multi.getAdminLang());
    }

    public static MultiJsonProducer unauthentified(Multi multi) {
        return new MultiJsonProducer(false, multi.getAdminMessageLocalisation());
    }

}
