/* FichothequeLib_Tools - Copyright (c) 2010-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.ExtractionSource;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.run.CorpusExtractResult;
import net.fichotheque.extraction.run.CorpusExtractor;
import net.fichotheque.extraction.run.ThesaurusExtractResult;
import net.fichotheque.extraction.run.ThesaurusExtractor;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.selection.CroisementCondition;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.selection.DocumentQueryBuilder;
import net.fichotheque.tools.selection.FicheQueryBuilder;
import net.fichotheque.tools.selection.IllustrationQueryBuilder;
import net.fichotheque.tools.selection.MotcleQueryBuilder;
import net.fichotheque.utils.CorpusUtils;
import net.fichotheque.utils.ExtractionUtils;
import net.fichotheque.utils.SelectionUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.fichotheque.utils.selection.SelectionContextBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;


/**
 *
 * @author Vincent Calame
 */
public final class ExtractionBuilderUtils {

    private ExtractionBuilderUtils() {
    }

    public static String testTagName(String tagName, String defaultTagName) {
        if (tagName == null) {
            return defaultTagName;
        }
        if (tagName.length() == 0) {
            return defaultTagName;
        }
        if (tagName.equals("NULL")) {
            return null;
        }
        if (testCharacters(tagName)) {
            return tagName;
        }
        return defaultTagName;
    }

    public static ExtractionSource getExtractionSource(Object dynamicObject) {
        return new SimpleExtractionSource(dynamicObject);
    }

    public static ExtractionSource getExtractionSource(Object dynamicObject, ExtractionContext extractionContext, ExtractionDef extractionDef) {
        List<CorpusExtractDef> corpusExtractDefList = extractionDef.getStaticCorpusExtractDefList();
        List<ThesaurusExtractDef> thesaurusExtractDefList = extractionDef.getStaticThesaurusExtractDefList();
        if ((corpusExtractDefList.isEmpty()) && (thesaurusExtractDefList.isEmpty())) {
            return new SimpleExtractionSource(dynamicObject);
        }
        List<CorpusExtractResult> corpusExtractResultList = runCorpusExtractDef(corpusExtractDefList, extractionContext);
        List<ThesaurusExtractResult> thesaurusExtractResultList = runThesaurusExtractDef(thesaurusExtractDefList, extractionContext);
        return new CompleteExtractionSource(dynamicObject, corpusExtractResultList, thesaurusExtractResultList);
    }

    public static CorpusExtractDef toCorpusExtractDef(ExtendedIncludeKey includeKey) {
        return toCorpusExtractDef(includeKey, ExtractionUtils.TITRE_FICHEFILTER);
    }

    public static CorpusExtractDef toCorpusExtractDef(ExtendedIncludeKey includeKey, FicheFilter ficheFilter) {
        if (!includeKey.getSubsetKey().isCorpusSubset()) {
            throw new IllegalArgumentException("Not a corpus subsetkey");
        }
        FicheQuery ficheQuery = FicheQueryBuilder.init()
                .addCorpus(includeKey.getSubsetKey())
                .toFicheQuery();
        CroisementCondition croisementCondition = SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter());
        return CorpusExtractDefBuilder.init(ficheFilter)
                .setName(includeKey.toString())
                .setBoolean(CorpusExtractDef.MASTER_BOOLEAN, includeKey.isMaster())
                .setEntryList(SelectionUtils.toFicheConditionEntry(ficheQuery, croisementCondition, false))
                .toCorpusExtractDef();
    }

    public static ThesaurusExtractDef toThesaurusExtractDef(ExtendedIncludeKey includeKey, Fichotheque fichotheque) {
        if (!includeKey.getSubsetKey().isThesaurusSubset()) {
            throw new IllegalArgumentException("Not a thesaurus subsetkey");
        }
        boolean withFicheStylePhrase = false;
        Subset subset = fichotheque.getSubset(includeKey.getSubsetKey());
        if (subset != null) {
            withFicheStylePhrase = ((Thesaurus) subset).getThesaurusMetadata().isWithFicheStyle();
        }
        MotcleFilter motcleFilter = MotcleFilterBuilder.init(MotcleFilter.DEFAULT_TYPE)
                .setWithLabels(true)
                .setWithFicheStylePhrase(withFicheStylePhrase)
                .toMotcleFilter();
        return toThesaurusExtractDef(includeKey, motcleFilter);
    }

    public static ThesaurusExtractDef toThesaurusExtractDef(ExtendedIncludeKey includeKey, MotcleFilter motcleFilter) {
        if (!includeKey.getSubsetKey().isThesaurusSubset()) {
            throw new IllegalArgumentException("Not a thesaurus subsetkey");
        }
        MotcleQuery motcleQuery = MotcleQueryBuilder.init()
                .addThesaurus(includeKey.getSubsetKey())
                .toMotcleQuery();
        CroisementCondition croisementCondition = SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter());
        return ThesaurusExtractDefBuilder.init(motcleFilter)
                .setName(includeKey.toString())
                .setBoolean(ThesaurusExtractDef.MASTER_BOOLEAN, includeKey.isMaster())
                .setEntryList(SelectionUtils.toMotcleConditionEntry(motcleQuery, croisementCondition, false))
                .toThesaurusExtractDef();
    }

    public static AlbumExtractDef toAlbumExtractDef(ExtendedIncludeKey includeKey) {
        return toAlbumExtractDef(includeKey, ExtractionUtils.DEFAULT_ILLUSTRATIONFILTER);
    }

    public static AlbumExtractDef toAlbumExtractDef(ExtendedIncludeKey includeKey, IllustrationFilter illustrationFilter) {
        if (!includeKey.getSubsetKey().isAlbumSubset()) {
            throw new IllegalArgumentException("Not a album subsetkey");
        }
        IllustrationQuery illustrationQuery = IllustrationQueryBuilder.init()
                .addAlbum(includeKey.getSubsetKey())
                .toIllustrationQuery();
        CroisementCondition croisementCondition = SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter());
        return AlbumExtractDefBuilder.init(illustrationFilter)
                .setName(includeKey.toString())
                .setEntryList(SelectionUtils.toIllustrationConditionEntry(illustrationQuery, croisementCondition))
                .toAlbumExtractDef();
    }

    public static AddendaExtractDef toAddendaExtractDef(ExtendedIncludeKey includeKey) {
        return toAddendaExtractDef(includeKey, ExtractionUtils.DEFAULT_DOCUMENTFILTER);
    }

    public static AddendaExtractDef toAddendaExtractDef(ExtendedIncludeKey includeKey, DocumentFilter documentFilter) {
        if (!includeKey.getSubsetKey().isAddendaSubset()) {
            throw new IllegalArgumentException("Not a addenda subsetkey");
        }
        DocumentQuery documentQuery = DocumentQueryBuilder.init()
                .addAddenda(includeKey.getSubsetKey())
                .toDocumentQuery();
        CroisementCondition croisementCondition = SelectionUtils.toCroisementCondition(includeKey.getMode(), includeKey.getPoidsFilter());
        return AddendaExtractDefBuilder.init(documentFilter)
                .setName(includeKey.toString())
                .setEntryList(SelectionUtils.toDocumentConditionEntry(documentQuery, croisementCondition))
                .toAddendaExtractDef();
    }


    public static List<CorpusExtractResult> runCorpusExtractDef(List<CorpusExtractDef> corpusExtractDefList, ExtractionContext extractionContext) {
        if (corpusExtractDefList.isEmpty()) {
            return ExtractionUtils.EMPTY_CORPUSEXTRACTRESULTLIST;
        }
        Fiches selectedFiches = extractionContext.getSelectedFiches();
        List<FicheMeta> bufferList = null;
        if (selectedFiches != null) {
            bufferList = CorpusUtils.toList(selectedFiches);
        }
        Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
        List<CorpusExtractResult> list = new ArrayList<CorpusExtractResult>();
        for (CorpusExtractDef corpusExtractDef : corpusExtractDefList) {
            SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                    .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                    .toSelectionContext();
            FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext)
                    .addAll(corpusExtractDef.getEntryList())
                    .toFicheSelector();
            List<Corpus> corpusList = ficheSelector.getCorpusList();
            if (corpusList.isEmpty()) {
                continue;
            }
            CorpusExtractor corpusExtractor = extractionContext.getExtractorProvider().getCorpusExtractor(corpusExtractDef);
            if (bufferList != null) {
                for (FicheMeta ficheMeta : bufferList) {
                    if (ficheSelector.test(ficheMeta)) {
                        corpusExtractor.add(ficheMeta, null);
                    }
                }
            } else {
                for (Corpus corpus : corpusList) {
                    for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                        if (ficheSelector.test(ficheMeta)) {
                            corpusExtractor.add(ficheMeta, null);
                        }
                    }
                }
            }
            CorpusExtractResult corpusExtractResult = corpusExtractor.getCorpusExtractResult();
            if (!corpusExtractResult.isEmpty()) {
                list.add(corpusExtractResult);
            }
        }
        int size = list.size();
        if (size == 0) {
            return ExtractionUtils.EMPTY_CORPUSEXTRACTRESULTLIST;
        }
        return list;
    }

    public static List<ThesaurusExtractResult> runThesaurusExtractDef(List<ThesaurusExtractDef> thesaurusExtractDefList, ExtractionContext extractionContext) {
        if (thesaurusExtractDefList.isEmpty()) {
            return ExtractionUtils.EMPTY_THESAURUSEXTRACTRESULTLIST;
        }
        List<ThesaurusExtractResult> list = new ArrayList<ThesaurusExtractResult>();
        SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, LocalisationUtils.getDefaultLang(extractionContext.getLangContext()))
                .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                .toSelectionContext();
        for (ThesaurusExtractDef thesaurusExtractDef : thesaurusExtractDefList) {
            MotcleSelector motcleSelector = MotcleSelectorBuilder.init(selectionContext).addAll(thesaurusExtractDef.getConditionEntryList()).toMotcleSelector();
            List<Thesaurus> thesaurusList = motcleSelector.getThesaurusList();
            if (thesaurusList.isEmpty()) {
                continue;
            }
            ThesaurusExtractor thesaurusExtractor = extractionContext.getExtractorProvider().getThesaurusExtractor(thesaurusExtractDef);
            for (Thesaurus thesaurus : thesaurusList) {
                for (Motcle motcle : thesaurus.getMotcleList()) {
                    if (motcleSelector.test(motcle)) {
                        thesaurusExtractor.add(motcle, null);
                    }
                }
            }
            ThesaurusExtractResult thesaurusExtractResult = thesaurusExtractor.getThesaurusExtractResult();
            if (!thesaurusExtractResult.getEntryList().isEmpty()) {
                list.add(thesaurusExtractResult);
            }
        }
        int size = list.size();
        if (size == 0) {
            return ExtractionUtils.EMPTY_THESAURUSEXTRACTRESULTLIST;
        }
        return list;
    }

    private static boolean testCharacters(String s) {
        int longueur = s.length();
        for (int i = 0; i < longueur; i++) {
            char carac = s.charAt(i);
            if (!isCharacter(carac)) {
                if (i == 0) {
                    return false;
                }
                if (!isNumber(carac)) {
                    if (i == (longueur - 1)) {
                        return false;
                    }
                    if (!isSpecialChar(carac)) {
                        return false;
                    }
                }

            }
        }
        return true;
    }

    private static boolean isCharacter(char carac) {
        return (((carac >= 'a') && (carac <= 'z')) || ((carac >= 'A') && (carac <= 'Z')));
    }

    private static boolean isNumber(char carac) {
        return ((carac >= '0') && (carac <= '9'));
    }

    private static boolean isSpecialChar(char carac) {
        switch (carac) {
            case '-':
            case '_':
                return true;
            default:
                return false;
        }
    }


    private static class SimpleExtractionSource implements ExtractionSource {

        private final Object dynamicObject;

        private SimpleExtractionSource(Object dynamicObject) {
            this.dynamicObject = dynamicObject;
        }

        @Override
        public Object getDynamicObject() {
            return dynamicObject;
        }

        @Override
        public List<CorpusExtractResult> getStaticCorpusExtractResultList() {
            return ExtractionUtils.EMPTY_CORPUSEXTRACTRESULTLIST;
        }

        @Override
        public List<ThesaurusExtractResult> getStaticThesaurusExtractResultList() {
            return ExtractionUtils.EMPTY_THESAURUSEXTRACTRESULTLIST;
        }

    }


    private static class CompleteExtractionSource implements ExtractionSource {

        private final Object dynamicObject;
        private final List<CorpusExtractResult> corpusExtractResultList;
        private final List<ThesaurusExtractResult> thesaurusExtractResultList;

        private CompleteExtractionSource(Object dynamicObject, List<CorpusExtractResult> corpusExtractResultList, List<ThesaurusExtractResult> thesaurusExtractResultList) {
            this.dynamicObject = dynamicObject;
            this.corpusExtractResultList = corpusExtractResultList;
            this.thesaurusExtractResultList = thesaurusExtractResultList;
        }

        @Override
        public Object getDynamicObject() {
            return dynamicObject;
        }

        @Override
        public List<CorpusExtractResult> getStaticCorpusExtractResultList() {
            return corpusExtractResultList;
        }

        @Override
        public List<ThesaurusExtractResult> getStaticThesaurusExtractResultList() {
            return thesaurusExtractResultList;
        }

    }

    public static List<CorpusExtractDef> wrap(CorpusExtractDef[] array) {
        return new CorpusExtractDefList(array);
    }


    private static class CorpusExtractDefList extends AbstractList<CorpusExtractDef> implements RandomAccess {

        private final CorpusExtractDef[] array;

        private CorpusExtractDefList(CorpusExtractDef[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public CorpusExtractDef get(int index) {
            return array[index];
        }

    }

    public static List<FicheRattachementDef> wrap(FicheRattachementDef[] array) {
        return new FicheRattachementDefList(array);
    }


    private static class FicheRattachementDefList extends AbstractList<FicheRattachementDef> implements RandomAccess {

        private final FicheRattachementDef[] array;

        private FicheRattachementDefList(FicheRattachementDef[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public FicheRattachementDef get(int index) {
            return array[index];
        }

    }

}
