/* FichothequeLib_Tools - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation.dom;

import java.text.ParseException;
import java.util.function.Consumer;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.importation.ThesaurusImport;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.importation.ImportationUtils;
import net.fichotheque.tools.importation.LiensImportBuilder;
import net.fichotheque.tools.importation.ThesaurusImportBuilder;
import net.mapeadores.util.attr.AttributeChangeBuilder;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LabelChangeBuilder;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusImportDOMReader {

    private final ThesaurusImportBuilder thesaurusImportBuilder;

    public ThesaurusImportDOMReader(ThesaurusImportBuilder thesaurusImportBuilder) {
        this.thesaurusImportBuilder = thesaurusImportBuilder;
    }

    public ThesaurusImportBuilder getThesaurusImportBuilder() {
        return thesaurusImportBuilder;
    }

    public void readThesaurusImport(Element element) {
        Consumer<Element> elementConsumer;
        String type = thesaurusImportBuilder.getType();
        if (type.equals(ThesaurusImport.CHANGE_TYPE)) {
            elementConsumer = new ChangeConsumer();
        } else if (type.equals(ThesaurusImport.CREATION_TYPE)) {
            elementConsumer = new CreationConsumer();
        } else if (type.equals(ThesaurusImport.MERGE_TYPE)) {
            elementConsumer = new MergeConsumer();
        } else if (type.equals(ThesaurusImport.MOVE_TYPE)) {
            elementConsumer = new MoveConsumer();
        } else if (type.equals(ThesaurusImport.REMOVE_TYPE)) {
            elementConsumer = new RemoveConsumer();
        } else {
            throw new SwitchException("Unknown type: " + type);
        }
        DOMUtils.readChildren(element, elementConsumer);
    }

    public static ThesaurusImportDOMReader fromRootElement(Fichotheque fichotheque, Element element) {
        String thesaurusName = element.getAttribute("thesaurus");
        try {
            SubsetKey thesaurusKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, thesaurusName);
            Thesaurus thesaurus = (Thesaurus) fichotheque.getSubset(thesaurusKey);
            if (thesaurus == null) {
                return null;
            }
            String type = element.getAttribute("type");
            if (!ImportationUtils.testThesaurusImportType(type)) {
                return null;
            }
            Thesaurus destinationThesaurus = null;
            if ((type.equals(ThesaurusImport.MERGE_TYPE)) || (type.equals(ThesaurusImport.MOVE_TYPE))) {
                String destinationThesaurusName = element.getAttribute("destination");
                try {
                    SubsetKey destinationSubsetKey = SubsetKey.parse(SubsetKey.CATEGORY_THESAURUS, destinationThesaurusName);
                    destinationThesaurus = (Thesaurus) fichotheque.getSubset(destinationSubsetKey);
                    if (thesaurus == null) {
                        return null;
                    }
                } catch (ParseException pe) {
                    return null;
                }
            }
            ThesaurusImportBuilder thesaurusImportBuilder = ThesaurusImportBuilder.newInstance(thesaurus, type, destinationThesaurus);
            return new ThesaurusImportDOMReader(thesaurusImportBuilder);
        } catch (ParseException pe) {
            return null;
        }
    }


    private class ChangeConsumer implements Consumer<Element> {

        private final ChangeMotcleConsumer changeMotcleConsumer = new ChangeMotcleConsumer();

        private ChangeConsumer() {
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("motcleimport")) {
                String idString = element.getAttribute("id");
                try {
                    int id = Integer.parseInt(idString);
                    Motcle motcle = thesaurusImportBuilder.getThesaurus().getMotcleById(id);
                    if (motcle != null) {
                        changeMotcleConsumer.clear();
                        DOMUtils.readChildren(element, changeMotcleConsumer);
                        thesaurusImportBuilder.addChange(motcle, changeMotcleConsumer.newIdalpha, changeMotcleConsumer.parent, changeMotcleConsumer.labelBuilder, changeMotcleConsumer.attributeChangeBuilder, changeMotcleConsumer.liensImportBuilder);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }

    }


    private class CreationConsumer implements Consumer<Element> {

        private final CreationMotcleConsumer creationMotcleConsumer = new CreationMotcleConsumer();

        private CreationConsumer() {
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("motcleimport")) {
                int newId = -1;
                String idString = element.getAttribute("id");
                try {
                    newId = Integer.parseInt(idString);
                } catch (NumberFormatException nfe) {
                }
                creationMotcleConsumer.clear();
                DOMUtils.readChildren(element, creationMotcleConsumer);
                thesaurusImportBuilder.addCreation(newId, creationMotcleConsumer.newIdalpha, creationMotcleConsumer.newParentId, creationMotcleConsumer.newParentIdalpha, creationMotcleConsumer.labelBuilder, creationMotcleConsumer.attributeChangeBuilder, creationMotcleConsumer.liensImportBuilder);
            }
        }

    }


    private class MergeConsumer implements Consumer<Element> {

        private MergeConsumer() {
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("motcleimport")) {
                String idString = element.getAttribute("id");
                String destinationString = element.getAttribute("destination");
                try {
                    int id = Integer.parseInt(idString);
                    int destinationId = Integer.parseInt(destinationString);
                    Motcle motcle = thesaurusImportBuilder.getThesaurus().getMotcleById(id);
                    Motcle destinationMotcle = thesaurusImportBuilder.getDestinationThesaurus().getMotcleById(destinationId);
                    if ((motcle != null) && (destinationMotcle != null)) {
                        thesaurusImportBuilder.addMerge(motcle, destinationMotcle);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }

    }


    private class MoveConsumer implements Consumer<Element> {

        private MoveConsumer() {
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("motcleimport")) {
                String idString = element.getAttribute("id");
                try {
                    int id = Integer.parseInt(idString);
                    Motcle motcle = thesaurusImportBuilder.getThesaurus().getMotcleById(id);
                    if (motcle != null) {
                        thesaurusImportBuilder.addMove(motcle);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }

    }


    private class RemoveConsumer implements Consumer<Element> {

        private RemoveConsumer() {
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("motcleimport")) {
                String idString = element.getAttribute("id");
                try {
                    int id = Integer.parseInt(idString);
                    Motcle motcle = thesaurusImportBuilder.getThesaurus().getMotcleById(id);
                    if (motcle != null) {
                        thesaurusImportBuilder.addRemove(motcle);
                    }
                } catch (NumberFormatException nfe) {
                }
            }
        }

    }


    private class ChangeMotcleConsumer implements Consumer<Element> {

        private final LiensImportDOMReader liensImportDOMReader;
        private LabelChangeBuilder labelBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private String newIdalpha;
        private Object parent;

        private ChangeMotcleConsumer() {
            this.liensImportDOMReader = new LiensImportDOMReader(thesaurusImportBuilder.getThesaurus().getFichotheque());
        }

        private void clear() {
            labelBuilder = new LabelChangeBuilder();
            attributeChangeBuilder = new AttributeChangeBuilder();
            liensImportBuilder = new LiensImportBuilder();
            newIdalpha = null;
            parent = null;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("attr")) {
                ImportationDomUtils.readAttrElement(element, attributeChangeBuilder);
            } else if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                ImportationDomUtils.readLabelElement(element, labelBuilder);
            } else if (tagName.equals("idalpha")) {
                CleanedString idalphaString = DOMUtils.contentToCleanedString(element);
                if (idalphaString != null) {
                    newIdalpha = idalphaString.toString();
                } else {
                    newIdalpha = null;
                }
            } else if (tagName.equals("parent")) {
                String idString = element.getAttribute("id");
                if (idString.length() == 0) {
                    parent = thesaurusImportBuilder.getThesaurus();
                } else {
                    try {
                        int parentId = Integer.parseInt(idString);
                        Motcle parentMotcle = thesaurusImportBuilder.getThesaurus().getMotcleById(parentId);
                        if (parentMotcle != null) {
                            parent = parentMotcle;
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
            } else {
                liensImportDOMReader.readLiensImport(liensImportBuilder, element);
            }
        }

    }


    private class CreationMotcleConsumer implements Consumer<Element> {

        private final LiensImportDOMReader liensImportDOMReader;
        private LabelChangeBuilder labelBuilder;
        private AttributeChangeBuilder attributeChangeBuilder;
        private LiensImportBuilder liensImportBuilder;
        private String newIdalpha;
        private int newParentId;
        private String newParentIdalpha;

        private CreationMotcleConsumer() {
            this.liensImportDOMReader = new LiensImportDOMReader(thesaurusImportBuilder.getThesaurus().getFichotheque());
        }

        private void clear() {
            labelBuilder = new LabelChangeBuilder();
            attributeChangeBuilder = new AttributeChangeBuilder();
            liensImportBuilder = new LiensImportBuilder();
            newIdalpha = null;
            newParentId = - 1;
            newParentIdalpha = null;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("attr")) {
                ImportationDomUtils.readAttrElement(element, attributeChangeBuilder);
            } else if ((tagName.equals("label")) || (tagName.equals("lib"))) {
                ImportationDomUtils.readLabelElement(element, labelBuilder);
            } else if (tagName.equals("idalpha")) {
                CleanedString idalphaString = DOMUtils.contentToCleanedString(element);
                if (idalphaString != null) {
                    newIdalpha = idalphaString.toString();
                } else {
                    newIdalpha = null;
                }
            } else if (tagName.equals("parent")) {
                String idString = element.getAttribute("id");
                if (idString.length() > 0) {
                    try {
                        newParentId = Integer.parseInt(idString);
                        if (newParentId < 1) {
                            newParentId = -1;
                        }
                    } catch (NumberFormatException nfe) {
                    }
                }
                if (newParentId == -1) {
                    String idalphaParent = element.getAttribute("idalpha");
                    if (idalphaParent.length() > 0) {
                        newParentIdalpha = idalphaParent;
                    }
                }
            } else {
                liensImportDOMReader.readLiensImport(liensImportBuilder, element);
            }
        }

    }

}
